package com.skybeacon.sdk.locate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import android.bluetooth.BluetoothDevice;

import com.skybeacon.sdk.utils.DataConvertUtils;
import com.skybeacon.sdk.utils.DefaultStaticValues;

/**
 * 扫描到的iBeacon数据的缓存、滤波处理等
 * 
 * @author frank
 * 
 */
public class SKYBeaconFilter {
	@SuppressWarnings("unused")
	private final String TAG = "SKYBeaconFilter";
	// 超时时间门限值
	private static long TIMEOUT_MILLISECOND_THRESHOLD = 5000;
	// 超时时间门限值
	private static long NEARBY_TIMEOUT_MILLISECOND_THRESHOLD = 8000;
	// 单id的iBeacon扫描结果
	private Map<String, SKYBeacon> ibeaconMap = new HashMap<String, SKYBeacon>();
	// 多id的iBeacon扫描结果
	private Map<String, SKYBeaconMultiIDs> ibeaconMultiIDsMap = new HashMap<String, SKYBeaconMultiIDs>();
	// 扫描到的原始数据
	private Map<BluetoothDevice, SKYBeaconScanRaw> ibeaconMapRaw = new HashMap<BluetoothDevice, SKYBeaconScanRaw>();
	private Map<String, LinkedList<SKYBeacon>> ibeaconNearbyUsed = new HashMap<String, LinkedList<SKYBeacon>>();
	// 定位使用的数据
	private List<SKYBeacon> locateBeaconList = new ArrayList<SKYBeacon>();
	private static SKYBeaconFilter instance = null;
	private Object lock = new Object();
	private int cacheTimeMillisecond = 5000;

	/**
	 * 单例获取接口
	 * 
	 * @return
	 */
	public synchronized static SKYBeaconFilter getInstance() {
		if (instance == null) {
			instance = new SKYBeaconFilter();
		}
		return instance;
	}

	public void setCacheTimeMillisecond(int cacheTimeMillisecond) {
		this.cacheTimeMillisecond = cacheTimeMillisecond;
	}

	/**
	 * 清除超时的iBeacon设备
	 */
	public void removeTimeoutBeacons() {
		long timestampNow = 0;
		timestampNow = DataConvertUtils.getCurrentTimestampMillisecond();
		synchronized (lock) {
			// 单id的ibeacon的过滤
			Iterator<Map.Entry<String, SKYBeacon>> it1 = ibeaconMap.entrySet().iterator();
			while (it1.hasNext()) {
				Map.Entry<String, SKYBeacon> entry = it1.next();
				SKYBeacon content = entry.getValue();
				if ((timestampNow - content.getTimestampMillisecond()) > TIMEOUT_MILLISECOND_THRESHOLD) {
					it1.remove();
				}
			}
			// 多id的ibeacon的过滤
			Iterator<Map.Entry<String, SKYBeaconMultiIDs>> it2 = ibeaconMultiIDsMap.entrySet().iterator();
			while (it2.hasNext()) {
				Map.Entry<String, SKYBeaconMultiIDs> entry = it2.next();
				SKYBeaconMultiIDs content = entry.getValue();
				if ((timestampNow - content.getTimestampMillisecond()) > TIMEOUT_MILLISECOND_THRESHOLD) {
					it2.remove();
				} else {
					Iterator<SKYBeacon> it3 = content.getBeaconList().iterator();
					while (it3.hasNext()) {
						SKYBeacon tmpBeacon = it3.next();
						if (timestampNow - tmpBeacon.getTimestampMillisecond() > TIMEOUT_MILLISECOND_THRESHOLD) {
							it3.remove();
						}
					}
				}
			}

			// nearby ibeacon
			Iterator<Map.Entry<String, LinkedList<SKYBeacon>>> it3 = ibeaconNearbyUsed.entrySet().iterator();
			while (it3.hasNext()) {
				Map.Entry<String, LinkedList<SKYBeacon>> entry = it3.next();
				int size = entry.getValue().size();
				long timestampDiff = 0;
				while (size > 0) {
					timestampDiff = timestampNow - entry.getValue().getLast().getTimestampMillisecond();
					if (timestampDiff > NEARBY_TIMEOUT_MILLISECOND_THRESHOLD) {
						entry.getValue().removeLast();
					} else {
						break;
					}
					size = entry.getValue().size();
				}
				if (entry.getValue().size() == 0) {
					it3.remove();
				}
			}
		}
	}

	/**
	 * 对数据进行解析和滤波处理
	 */
	public void filterBeacons() {
		try {
			Iterator<Map.Entry<BluetoothDevice, SKYBeaconScanRaw>> it = ibeaconMapRaw.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<BluetoothDevice, SKYBeaconScanRaw> entry = it.next();
				final BluetoothDevice key = entry.getKey();
				final SKYBeaconScanRaw content = entry.getValue();
				if (key != null) {
					SKYBeaconMultiIDs skyBeaconMultiIDs = new SKYBeaconMultiIDs();
					final SKYBeacon skyBeacon = SKYBeaconScanProtocol.formatToSKYBeacon(content, skyBeaconMultiIDs);
					if (skyBeacon != null) {
						if (skyBeacon.getRssi() < 0) {
							locateBeaconList.add(skyBeacon);
							addDeviceNearby(skyBeacon);
							if (skyBeaconMultiIDs.isSeekcyBeacon() == 1) {
								addDeviceMultiIDs(skyBeaconMultiIDs, skyBeacon);
							} else {
								addDeviceSingleID(skyBeacon);
							}
						}
					}
				}
			}
			// 注意使用完要清空
			clearMapRaw();
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	/**
	 * 获取单id ibeaconList数据
	 * 
	 * @return
	 */
	public Map<String, SKYBeacon> getBeaconMapList() {
		Map<String, SKYBeacon> ret = new HashMap<String, SKYBeacon>();
		synchronized (lock) {
			ret.putAll(ibeaconMap);
		}
		return ret;
	}

	/**
	 * 获取多id ibeaconList数据
	 * 
	 * @return
	 */
	public Map<String, SKYBeaconMultiIDs> getBeaconMultiIDsMapList() {
		Map<String, SKYBeaconMultiIDs> ret = new HashMap<String, SKYBeaconMultiIDs>();
		synchronized (lock) {
			ret.putAll(ibeaconMultiIDsMap);
		}
		return ret;
	}

	/**
	 * 获取单id ibeaconList数据
	 * 
	 * @return
	 */
	public List<SKYBeacon> getBeaconList() {
		// List<SKYBeacon> ret = null;
		// synchronized (lock) {
		// ret = new ArrayList<SKYBeacon>(ibeaconMap.values());
		// }
		// return ret;
		List<SKYBeacon> ret = new ArrayList<SKYBeacon>();
		long timestampMillisecond = DataConvertUtils.getCurrentTimestampMillisecond();
		synchronized (lock) {
			Iterator<Map.Entry<String, SKYBeacon>> it = ibeaconMap.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, SKYBeacon> entry = it.next();
				final String key = entry.getKey();
				final SKYBeacon content = entry.getValue();
				if (key != null) {
					long timestampDiff = timestampMillisecond - content.getTimestampMillisecond();
					if (timestampDiff < this.cacheTimeMillisecond) {
						ret.add(content);
					}
				}
			}
		}
		return ret;
	}

	/**
	 * 获取多id ibeaconList数据
	 * 
	 * @return
	 */
	public List<SKYBeaconMultiIDs> getBeaconMultiIDsList() {
		// List<SKYBeaconMultiIDs> ret = null;
		// synchronized (lock) {
		// ret = new ArrayList<SKYBeaconMultiIDs>(ibeaconMultiIDsMap.values());
		// }
		// return ret;
		List<SKYBeaconMultiIDs> ret = new ArrayList<SKYBeaconMultiIDs>();
		long timestampMillisecond = DataConvertUtils.getCurrentTimestampMillisecond();
		synchronized (lock) {
			Iterator<Map.Entry<String, SKYBeaconMultiIDs>> it = ibeaconMultiIDsMap.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, SKYBeaconMultiIDs> entry = it.next();
				final String key = entry.getKey();
				final SKYBeaconMultiIDs content = entry.getValue();
				if (key != null) {
					long timestampDiff = timestampMillisecond - content.getTimestampMillisecond();
					if (timestampDiff < this.cacheTimeMillisecond) {
						ret.add(content);
					}
				}
			}
		}
		return ret;
	}

	/**
	 * 获取单id ibeaconList数据
	 * 
	 * @return
	 */
	public List<SKYBeacon> getBeaconList(SKYRegion skyBeaconRegion) {
		List<SKYBeacon> ret = new ArrayList<SKYBeacon>();
		synchronized (lock) {
			Iterator<Map.Entry<String, SKYBeacon>> it = ibeaconMap.entrySet().iterator();
			long timestampMillisecond = DataConvertUtils.getCurrentTimestampMillisecond();
			while (it.hasNext()) {
				Map.Entry<String, SKYBeacon> entry = it.next();
				final String key = entry.getKey();
				final SKYBeacon content = entry.getValue();
				if (key != null) {
					if (!isFilter(content, skyBeaconRegion)) {
						long timestampDiff = timestampMillisecond - content.getTimestampMillisecond();
						if (timestampDiff < this.cacheTimeMillisecond) {
							ret.add(content);
						}
					}
				}
			}
		}
		return ret;
	}

	/**
	 * 获取多id ibeaconList数据
	 * 
	 * @return
	 */
	public List<SKYBeaconMultiIDs> getBeaconMultiIDsList(SKYRegion skyBeaconRegion) {
		List<SKYBeaconMultiIDs> ret = new ArrayList<SKYBeaconMultiIDs>();
		synchronized (lock) {
			Iterator<Map.Entry<String, SKYBeaconMultiIDs>> it = ibeaconMultiIDsMap.entrySet().iterator();
			long timestampMillisecond = DataConvertUtils.getCurrentTimestampMillisecond();
			while (it.hasNext()) {
				Map.Entry<String, SKYBeaconMultiIDs> entry = it.next();
				final String key = entry.getKey();
				final SKYBeaconMultiIDs content = entry.getValue();
				if (key != null) {
					if (!isFilter(content, skyBeaconRegion)) {
						long timestampDiff = timestampMillisecond - content.getTimestampMillisecond();
						if (timestampDiff < this.cacheTimeMillisecond) {
							ret.add(content);
						}
					}
				}
			}
		}
		return ret;
	}

	/**
	 * 获取定位数据
	 * 
	 * @return
	 */
	public List<SKYBeacon> getLocateBeaconList() {
		List<SKYBeacon> result = new ArrayList<SKYBeacon>();
		synchronized (locateBeaconList) {
			result.addAll(locateBeaconList);
			locateBeaconList.clear();
		}
		return result;
	}

	/**
	 * 获取单id ibeaconList数据
	 * 
	 * @return
	 */
	public Map<String, LinkedList<SKYBeacon>> getNearbyBeaconMap(SKYRegion skyBeaconRegion) {
		Map<String, LinkedList<SKYBeacon>> ret = new HashMap<String, LinkedList<SKYBeacon>>();
		synchronized (lock) {
			Iterator<Map.Entry<String, LinkedList<SKYBeacon>>> it = ibeaconNearbyUsed.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry<String, LinkedList<SKYBeacon>> entry = it.next();
				final String key = entry.getKey();
				final LinkedList<SKYBeacon> content = entry.getValue();
				if (key != null && content.size() > 0) {
					if (!isFilter(content.get(0), skyBeaconRegion)) {
						ret.put(content.get(0).getDeviceAddress(), content);
					}
				}
			}
		}
		return ret;
	}

	/**
	 * 获取nearby原始数据
	 * 
	 * @return
	 */
	public Map<String, LinkedList<SKYBeacon>> getNearbyBeaconMap() {
		Map<String, LinkedList<SKYBeacon>> ret = new HashMap<String, LinkedList<SKYBeacon>>();
		synchronized (lock) {
			ret.putAll(ibeaconNearbyUsed);
		}
		return ret;
	}

	/**
	 * 添加设备
	 * 
	 * @param bluetoothDevice
	 * @param rssi
	 * @param scanRecord
	 */
	public void addDevice(final BluetoothDevice bluetoothDevice, final int rssi, final byte[] scanRecord) {
		// 不加锁的原因是提高效率，在外围顺序执行来避免线程安全问题
		SKYBeaconScanRaw scanRaw = new SKYBeaconScanRaw(bluetoothDevice, scanRecord, rssi);
		ibeaconMapRaw.put(bluetoothDevice, scanRaw);
	}

	/**
	 * 清空数据
	 */
	public void clear() {
		clearMap();
		clearMapRaw();
	}

	/**
	 * 清空滤波后的iBeacon数据
	 */
	public void clearMap() {
		synchronized (lock) {
			ibeaconMap.clear();
			ibeaconMultiIDsMap.clear();
			locateBeaconList.clear();
		}
	}

	/**
	 * 清空铝箔后的iBeacon数据
	 */
	public void clearRangeMap() {
		synchronized (lock) {
			ibeaconMap.clear();
			ibeaconMultiIDsMap.clear();
		}
	}

	/**
	 * 晴空原始数据
	 */
	public void clearMapRaw() {
		ibeaconMapRaw.clear();
	}

	private SKYBeaconFilter() {

	}

	private void addDeviceSingleID(SKYBeacon skyBeacon) {
		if (skyBeacon == null) {
			return;
		}

		SKYBeacon last = null;
		last = ibeaconMap.get(skyBeacon.getDeviceAddress());
		if (last != null) {
			long timestampDiff = skyBeacon.getTimestampMillisecond() - last.getTimestampMillisecond();
			if (timestampDiff < 3000) {
				skyBeacon.setRssi((skyBeacon.getRssi() + last.getRssi()) / 2);
				skyBeacon.setDistance(SKYBeaconScanProtocol.solveDistance(skyBeacon.getMeasuredPower(), skyBeacon.getRssi()));
			}
			synchronized (lock) {
				ibeaconMap.put(skyBeacon.getDeviceAddress(), skyBeacon);
			}
		} else {
			synchronized (lock) {
				ibeaconMap.put(skyBeacon.getDeviceAddress(), skyBeacon);
			}
		}
	}

	private void addDeviceMultiIDs(SKYBeaconMultiIDs skyBeaconMultiIDs, SKYBeacon skyBeacon) {
		if (skyBeaconMultiIDs == null) {
			return;
		}
		if (skyBeaconMultiIDs.isSeekcyBeacon() == 1) {
			synchronized (lock) {
				if (ibeaconMultiIDsMap.containsKey(skyBeaconMultiIDs.getDeviceAddress())) {
					SKYBeaconMultiIDs skyBeaconMultiIDsTmp = ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress());
					List<SKYBeacon> beaconListTmp = skyBeaconMultiIDsTmp.getBeaconList();
					List<SKYBeacon> beaconListNewTmp = new ArrayList<SKYBeacon>();
					if (beaconListTmp != null && beaconListTmp.size() != 0) {
						boolean flag = false;
						for (int i = 0; i < beaconListTmp.size(); i++) {
							if (beaconListTmp.get(i).getDeviceAddress().equals(skyBeacon.getDeviceAddress())) {
								flag = true;
								long timestampDiff = skyBeacon.getTimestampMillisecond() - beaconListTmp.get(i).getTimestampMillisecond();
								if (timestampDiff < 3000) {
									skyBeacon.setRssi((skyBeacon.getRssi() + beaconListTmp.get(i).getRssi()) / 2);
									skyBeacon.setDistance(SKYBeaconScanProtocol.solveDistance(skyBeacon.getMeasuredPower(), skyBeacon.getRssi()));
								}
								beaconListNewTmp.add(skyBeacon);
							} else {
								beaconListNewTmp.add(beaconListTmp.get(i));
							}
						}
						if (!flag) {
							beaconListNewTmp.add(skyBeacon);
						}
					} else {
						beaconListNewTmp.add(skyBeacon);
					}
					ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setTimestampMillisecond(skyBeaconMultiIDs.getTimestampMillisecond());
					ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setBeaconList(beaconListNewTmp);
					if (!skyBeaconMultiIDs.getDeviceName().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_NAME)) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setDeviceName(skyBeaconMultiIDs.getDeviceName());
					}
					if (skyBeaconMultiIDs.getHardwareVersion() != DefaultStaticValues.DEFAULT_SKY_BEACON_HARDWARE_VERSION) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setHardwareVersion(skyBeaconMultiIDs.getHardwareVersion());
					}
					if (skyBeaconMultiIDs.getFirmwareVersionMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MAJOR) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setFirmwareVersionMajor(skyBeaconMultiIDs.getFirmwareVersionMajor());
					}
					if (skyBeaconMultiIDs.getFirmwareVersionMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_FIRMWARE_VERSION_MINOR) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setFirmwareVersionMinor(skyBeaconMultiIDs.getFirmwareVersionMinor());
					}
					if (skyBeaconMultiIDs.getIntervalMillisecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_INTERVAL_MILLISECOND_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setIntervalMillisecond(skyBeaconMultiIDs.getIntervalMillisecond());
					}
					if (skyBeaconMultiIDs.getBattery() != DefaultStaticValues.DEFAULT_SKY_BEACON_BATTERY_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setBattery(skyBeaconMultiIDs.getBattery());
					}
					if (skyBeaconMultiIDs.getTemperature() != DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setTemperature(skyBeaconMultiIDs.getTemperature());
					}
					if (skyBeaconMultiIDs.getTemperatureUpdateSecond() != DefaultStaticValues.DEFAULT_SKY_BEACON_TEMPERATURE_UPDATE_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setTemperatureUpdateSecond(skyBeaconMultiIDs.getTemperatureUpdateSecond());
					}
					if (skyBeaconMultiIDs.isLocked() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_LOCKED_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setLocked(skyBeaconMultiIDs.isLocked());
					}
					if (skyBeaconMultiIDs.isEncrypted() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_ENCRYPTED_FALSE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setEncrypted(skyBeaconMultiIDs.isEncrypted());
					}
					if (skyBeaconMultiIDs.isSeekcyBeacon() != DefaultStaticValues.DEFAULT_SKY_BEACON_IS_SEEKCY_IBEACON) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setSeekcyBeacon(skyBeaconMultiIDs.isSeekcyBeacon());
					}
					if (skyBeaconMultiIDs.getLedState() != DefaultStaticValues.DEFAULT_SKY_BEACON_LED_STATE) {
						ibeaconMultiIDsMap.get(skyBeaconMultiIDs.getDeviceAddress()).setLedState(skyBeaconMultiIDs.getLedState());
					}
				} else {
					if (skyBeaconMultiIDs.getHardwareVersion() != DefaultStaticValues.DEFAULT_SKY_BEACON_HARDWARE_VERSION) {
						skyBeaconMultiIDs.addBeaconList(skyBeacon);
						ibeaconMultiIDsMap.put(skyBeaconMultiIDs.getDeviceAddress(), skyBeaconMultiIDs);
					}
				}
			}
		}
	}

	public void addDeviceNearby(SKYBeacon skyBeacon) {
		if (skyBeacon == null) {
			return;
		}
		synchronized (lock) {
			if (ibeaconNearbyUsed.containsKey(skyBeacon.getDeviceAddress())) {
				LinkedList<SKYBeacon> last = ibeaconNearbyUsed.get(skyBeacon.getDeviceAddress());
				if (last != null) {
					long timestampDiff = skyBeacon.getTimestampMillisecond() - last.getFirst().getTimestampMillisecond();
					if (timestampDiff < 3000) {
						int tmpRssi = (int) (skyBeacon.getRssi() * 0.6 + last.getFirst().getRssi() * 0.4);
						// skyBeacon.setRssi((skyBeacon.getRssi() +
						// last.getFirst().getRssi()) / 2);
						// if
						// (skyBeacon.getDeviceAddress().equals("19:18:FC:00:7D:63"))
						// {
						// Log.d(TAG,
						// "Device Address: 19:18:FC:00:7D:63, Rssi Before: " +
						// String.valueOf(last.getFirst().getRssi()) +
						// ", Rssi This: " + String.valueOf(skyBeacon.getRssi())
						// + ", Rssi Filter: " + String.valueOf(tmpRssi));
						// }
						skyBeacon.setRssi(tmpRssi);
						skyBeacon.setDistance(SKYBeaconScanProtocol.solveDistance(skyBeacon.getMeasuredPower(), skyBeacon.getRssi()));
					}
					ibeaconNearbyUsed.get(skyBeacon.getDeviceAddress()).addFirst(skyBeacon);
				} else {
					LinkedList<SKYBeacon> tmp = new LinkedList<SKYBeacon>();
					tmp.addFirst(skyBeacon);
					ibeaconNearbyUsed.put(skyBeacon.getDeviceAddress(), tmp);
				}
			} else {
				LinkedList<SKYBeacon> tmp = new LinkedList<SKYBeacon>();
				tmp.addFirst(skyBeacon);
				ibeaconNearbyUsed.put(skyBeacon.getDeviceAddress(), tmp);
			}
		}
	}

	private boolean isFilter(SKYBeacon skyBeacon, SKYRegion skyBeaconRegion) {
		if (skyBeaconRegion != null) {
			if (!skyBeaconRegion.getDeviceAddress().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS) && !skyBeaconRegion.getDeviceAddress().equals(skyBeacon.getDeviceAddress())) {
				return true;
			}
			if (!skyBeaconRegion.getProximityUUID().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID) && !skyBeaconRegion.getProximityUUID().equals(skyBeacon.getProximityUUID())) {
				return true;
			}
			if (skyBeaconRegion.getMajor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE && skyBeaconRegion.getMajor() != skyBeacon.getMajor()) {
				return true;
			}
			if (skyBeaconRegion.getMinor() != DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE && skyBeaconRegion.getMinor() != skyBeacon.getMinor()) {
				return true;
			}
		}

		return false;
	}

	private boolean isFilter(SKYBeaconMultiIDs skyBeaconMultiIDs, SKYRegion skyBeaconRegion) {
		if (skyBeaconRegion != null) {
			if (!skyBeaconRegion.getDeviceAddress().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS) && !skyBeaconRegion.getDeviceAddress().equals(skyBeaconMultiIDs.getDeviceAddress())) {
				return true;
			}
		}

		return false;
	}
}
