package com.skybeacon.sdk.locate;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Timer;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;

import com.skybeacon.sdk.MonitoringBeaconsListener;
import com.skybeacon.sdk.RangingBeaconsListener;
import com.skybeacon.sdk.ScanServiceStateCallback;
import com.skybeacon.sdk.decryption.DecryptProcess;
import com.skybeacon.sdk.utils.DataConvertUtils;
import com.skybeacon.sdk.utils.DefaultStaticValues;

public class SKYBeaconManager {
	private static SKYBeaconManager instance = null;
	private Context context = null;
	private Timer timerScan;
	private Timer timerMonitor;
	private ScanServiceStateCallback scanServiceStateCallback;
	private boolean isScanServiceBind = false;
	private boolean isRangingBeaconsStart = false;
	private boolean isMonitorBeaconsStart = false;
	private RangingBeaconsListener rangingBeaconsListener;
	private SKYRegion scanBeaconRegion;
	private int cacheTimeMillisecond = 5000;
	private int scanTimerIntervalMillisecond = 1200;

	private MonitoringBeaconsListener monitoringBeaconsListener;
	private MonitorBeaconsProcess monitorBeaconsProcess;
	private NearbyLogicProcess nearbyLogicProcess;

	/**
	 * 构造函数
	 * 
	 * @param context
	 */
	public synchronized static SKYBeaconManager getInstance() {
		if (instance == null) {
			instance = new SKYBeaconManager();
		}
		return instance;
	}

	/**
	 * App的注册， Application中调用该方法
	 * 
	 * @param context
	 */
	public void init(Context context) {
		this.context = context;
		PublicStaticValues.init(context);
	}

	/**
	 * 设置数据缓存时间，单位毫秒，默认缓存5000毫秒的数据
	 * 
	 * @param cacheTimeMillisecond
	 */
	public void setCacheTimeMillisecond(int cacheTimeMillisecond) {
		this.cacheTimeMillisecond = cacheTimeMillisecond;
		SKYBeaconFilter.getInstance().setCacheTimeMillisecond(this.cacheTimeMillisecond);
	}

	/**
	 * 设置返回数据时间间隔，单位毫秒，默认为1200毫秒返回一次
	 * 
	 * @param scanTimerIntervalMillisecond
	 */
	public void setScanTimerIntervalMillisecond(int scanTimerIntervalMillisecond) {
		this.scanTimerIntervalMillisecond = scanTimerIntervalMillisecond;
		timerScan.cancel();
		timerScan = new Timer();
		timerScan.scheduleAtFixedRate(new TimerTaskRanging(), 10, this.scanTimerIntervalMillisecond);
	}

	/**
	 * 
	 * @param threshold
	 */
	public void setNearbyTriggerThreshold(SKYBeaconNearbyThreshold threshold) {
		nearbyLogicProcess.setSkyBeaconNearbyThreshold(threshold);
	}

	/**
	 * 
	 * @param rssiOneMeter
	 */
	public void setPhoneMeasuredPower(int rssiOneMeter) {
		nearbyLogicProcess.setPhoneMeasuredPower(rssiOneMeter);
	}

	/**
	 * 清空缓存数据
	 */
	public void clearCacheDatas() {
		SKYBeaconFilter.getInstance().clearRangeMap();
	}

	/**
	 * 设置防蹭用密钥
	 * 
	 * @param key
	 *            ：向seekcy申请，长度为64个字符的String
	 * @return 是否设置成功
	 */
	public boolean setBroadcastKey(String key) {
		boolean result = false;
		if (key != null && key.length() == 64) {
			result = true;
			DecryptProcess.init(DataConvertUtils.hexStringToByte(key));
		}
		if (key != null && key.equals("")) {
			result = true;
			DecryptProcess.init(DataConvertUtils.hexStringToByte("0000000000000000000000000000000000000000000000000000000000000000"));
		}
		return result;
	}

	/**
	 * 判断手机是否支持Ble4.0
	 * 
	 * @return 是或者否
	 */
	public boolean isSupportBLE() {
		return SKYBeaconScanUnit.getInstance().isSuportBLE();
	}

	/**
	 * 判断手机蓝牙是否开启
	 * 
	 * @return 是或者否
	 */
	public boolean isBluetoothEnable() {
		return SKYBeaconScanUnit.getInstance().isBluetoothEnabled();
	}

	/**
	 * 开启扫描服务
	 * 
	 * @param stateCallback
	 *            扫描服务开启成功与否的回调函数
	 */
	public void startScanService(ScanServiceStateCallback stateCallback) {
		System.out.println("Start Scan Service!");
		scanServiceStateCallback = stateCallback;
		Intent intent = new Intent(context, BleScanService.class);
		isScanServiceBind = context.bindService(intent, connection, Context.BIND_AUTO_CREATE);
		// timerScan = new Timer();
		// timerScan.scheduleAtFixedRate(new TimerTaskRanging(), 10, 1200);
		// // monitor
		// monitorBeaconsProcess = new MonitorBeaconsProcess();
		// timerMonitor = new Timer();
		// timerMonitor.scheduleAtFixedRate(new TimerTaskMonitoring(), 10,
		// 3000);
	}

	/**
	 * 关闭扫描服务
	 */
	public void stopScanService() {
		System.out.println("Stop Scan Service!");
		if (isScanServiceBind) {
			context.unbindService(connection);
			isScanServiceBind = false;
			System.out.println("isScanServiceBind!");
		}
		if (timerScan != null) {
			// timerScan.cancel();
			// System.out.println("Stop timerScan!");
		}
		if (timerMonitor != null) {
			// timerMonitor.cancel();
			// System.out.println("Stop timerMonitor!");
		}
	}

	/**
	 * 开启扫描SKYBeacon
	 * 
	 * @param beaconRegion
	 *            扫描约束
	 */
	public void startRangingBeacons(SKYRegion beaconRegion) {
		scanBeaconRegion = beaconRegion;
		isRangingBeaconsStart = true;
	}

	/**
	 * 设置扫描监听回调
	 * 
	 * @param rangingBeaconsListener
	 */
	public void setRangingBeaconsListener(RangingBeaconsListener rangingBeaconsListener) {
		this.rangingBeaconsListener = rangingBeaconsListener;
	}

	/**
	 * 关闭扫描SKYBeacon
	 * 
	 * @param beaconRegion
	 *            扫描约束
	 */
	public void stopRangingBeasons(SKYRegion beaconRegion) {
		scanBeaconRegion = null;
		isRangingBeaconsStart = false;
	}

	/**
	 * 开启监听SKYBeacon
	 * 
	 * @param beaconRegion
	 *            监听约束
	 */
	public void startMonitoringBeacons(SKYRegion beaconRegion) {
		isMonitorBeaconsStart = true;
		monitorBeaconsProcess.addSKYBeaconRegion(beaconRegion);
	}

	/**
	 * 设置区域监听回调
	 * 
	 * @param monitoringBeaconsListener
	 */
	public void setMonitoringBeaconsListener(MonitoringBeaconsListener monitoringBeaconsListener) {
		this.monitoringBeaconsListener = monitoringBeaconsListener;
	}

	/**
	 * 关闭监听SKYBeacon
	 * 
	 * @param beaconRegion
	 *            监听约束
	 */
	public void stopMonitoringBeacons(SKYRegion beaconRegion) {
		monitorBeaconsProcess.removeSKYBeaconRegion(beaconRegion);
		if (monitorBeaconsProcess.getSKYBeaconRegionCount() == 0) {
			isMonitorBeaconsStart = false;
		}
	}

	/**
	 * Service链接状态回调函数
	 */
	private ServiceConnection connection = new ServiceConnection() {

		@Override
		public void onServiceConnected(ComponentName arg0, IBinder arg1) {
			// TODO Auto-generated method stub
			scanServiceStateCallback.onServiceConnected();
			System.out.println("Scan Service Connected!");
		}

		@Override
		public void onServiceDisconnected(ComponentName arg0) {
			// TODO Auto-generated method stub
			scanServiceStateCallback.onServiceDisconnected();
			System.out.println("Scan Service Disconnected!");
		}
	};

	/**
	 * 定时器，扫描回调
	 * 
	 * @author frank
	 * 
	 */
	private class TimerTaskRanging extends java.util.TimerTask {

		List<SKYBeacon> beaconList = null;
		List<SKYBeaconMultiIDs> beaconMultiIDsList = null;
		List<SKYBeacon> beaconNearbyList = null;
		Map<String, LinkedList<SKYBeacon>> beaconNearbyMap = null;

		@Override
		public void run() {
			// TODO Auto-generated method stub
			if (isRangingBeaconsStart) {
				if (beaconList != null) {
					beaconList.clear();
				}
				beaconList = null;
				if (beaconMultiIDsList != null) {
					beaconMultiIDsList.clear();
				}
				beaconMultiIDsList = null;
				if (beaconNearbyList != null) {
					beaconNearbyList.clear();
				}
				beaconNearbyList = null;
				if (beaconNearbyMap != null) {
					beaconNearbyMap.clear();
				}
				beaconNearbyMap = null;
				if (scanBeaconRegion == null) {
					beaconList = SKYBeaconFilter.getInstance().getBeaconList();
					beaconMultiIDsList = SKYBeaconFilter.getInstance().getBeaconMultiIDsList();
					beaconNearbyMap = SKYBeaconFilter.getInstance().getNearbyBeaconMap();
				} else {
					if (scanBeaconRegion.getIdentifier().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_REGION_IDENTIFIER)
							&& scanBeaconRegion.getDeviceAddress().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_DEVICE_ADDRESS)
							&& scanBeaconRegion.getProximityUUID().equals(DefaultStaticValues.DEFAULT_SKY_BEACON_PROXIMITY_UUID)
							&& scanBeaconRegion.getMajor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MAJOR_FALSE && scanBeaconRegion.getMinor() == DefaultStaticValues.DEFAULT_SKY_BEACON_MINOR_FALSE) {
						beaconList = SKYBeaconFilter.getInstance().getBeaconList();
						beaconMultiIDsList = SKYBeaconFilter.getInstance().getBeaconMultiIDsList();
						beaconNearbyMap = SKYBeaconFilter.getInstance().getNearbyBeaconMap();
					} else {
						beaconList = SKYBeaconFilter.getInstance().getBeaconList(scanBeaconRegion);
						beaconMultiIDsList = SKYBeaconFilter.getInstance().getBeaconMultiIDsList(scanBeaconRegion);
						beaconNearbyMap = SKYBeaconFilter.getInstance().getNearbyBeaconMap(scanBeaconRegion);
					}
				}

				if (beaconNearbyMap != null && beaconNearbyMap.size() >= 0) {
					beaconNearbyList = nearbyLogicProcess.processNearbyLogic(beaconNearbyMap);
					if (beaconNearbyList != null && beaconNearbyList.size() >= 0) {
						rangingBeaconsListener.onRangedNearbyBeacons(scanBeaconRegion, beaconNearbyList);
					}
				}

				if (beaconList != null && beaconList.size() >= 0) {
					rangingBeaconsListener.onRangedBeacons(scanBeaconRegion, beaconList);
				}
				if (beaconMultiIDsList != null && beaconMultiIDsList.size() >= 0) {
					rangingBeaconsListener.onRangedBeaconsMultiIDs(scanBeaconRegion, beaconMultiIDsList);
				}
			}
		}
	}

	/**
	 * 定时器，监听回调
	 * 
	 * @author frank
	 * 
	 */
	private class TimerTaskMonitoring extends java.util.TimerTask {

		List<SKYBeacon> beaconList = null;
		List<SKYBeaconMultiIDs> beaconMultiIDsList = null;
		List<SKYBeacon> beaconListReturn = new ArrayList<SKYBeacon>();
		List<Integer> states = new ArrayList<Integer>();
		List<SKYRegion> skyRegions = new ArrayList<SKYRegion>();

		@Override
		public void run() {
			// TODO Auto-generated method stub
			if (isMonitorBeaconsStart) {
				if (beaconList != null) {
					beaconList.clear();
				}
				beaconList = null;
				if (beaconMultiIDsList != null) {
					beaconMultiIDsList.clear();
				}
				beaconMultiIDsList = null;
				if (beaconListReturn != null) {
					beaconListReturn.clear();
				}
				if (states != null) {
					states.clear();
				}
				if (skyRegions != null) {
					skyRegions.clear();
				}
				beaconList = SKYBeaconFilter.getInstance().getBeaconList();
				beaconMultiIDsList = SKYBeaconFilter.getInstance().getBeaconMultiIDsList();
				beaconListReturn.addAll(beaconList);

				states = monitorBeaconsProcess.getBeaconsStates(beaconList, beaconMultiIDsList, skyRegions);

				for (int i = 0; i < beaconMultiIDsList.size(); ++i) {
					for (int j = 0; j < beaconMultiIDsList.get(i).getBeaconList().size(); ++j) {
						beaconListReturn.add(beaconMultiIDsList.get(i).getBeaconList().get(j));
					}
				}
				if (states != null && states.size() != 0 && skyRegions != null && skyRegions.size() != 0 && skyRegions.size() == states.size()) {
					for (int i = 0; i < skyRegions.size(); ++i) {
						if (states.get(i) == MonitorBeaconsProcess.STATE_ENTERED_INIT) {
							monitoringBeaconsListener.onEnteredRegion(skyRegions.get(i), beaconListReturn);
						} else if (states.get(i) == MonitorBeaconsProcess.STATE_EXITED_INIT) {
							monitoringBeaconsListener.onExitedRegion(skyRegions.get(i), beaconListReturn);
						}
					}
				}
			}
		}
	}

	/**
	 * 构造函数
	 */
	private SKYBeaconManager() {
		timerScan = new Timer();
		timerScan.scheduleAtFixedRate(new TimerTaskRanging(), 10, 1200);
		// monitor
		monitorBeaconsProcess = new MonitorBeaconsProcess();
		timerMonitor = new Timer();
		timerMonitor.scheduleAtFixedRate(new TimerTaskMonitoring(), 10, 3000);
		nearbyLogicProcess = new NearbyLogicProcess();
	}
}
